package org.jeecg.modules.archives.controller;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import com.google.common.collect.Lists;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.NotDuplicate;
import org.jeecg.common.dto.archives.ExpensePriceReqDTO;
import org.jeecg.common.dto.archives.ExpensePriceRespDTO;
import org.jeecg.common.dto.archives.ProArchivesAddReqDTO;
import org.jeecg.common.dto.archives.ProArchivesAddRespDTO;
import org.jeecg.common.dto.archives.ProPlanListReqDTO;
import org.jeecg.common.dto.archives.ProPlanListRespDTO;
import org.jeecg.common.dto.basedata.ArchivesBusinessListReqDTO;
import org.jeecg.common.dto.basedata.ArchivesBusinessListRespDTO;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.modules.archives.entity.ProjectManageArchives;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.modules.archives.entity.ProjectManageArchivesAndBusiness;
import org.jeecg.modules.archives.entity.ProjectManageBusinessOther;
import org.jeecg.modules.archives.service.ProjectManageArchivesAndBusinessService;
import org.jeecg.modules.archives.service.ProjectManageArchivesService;
import org.jeecg.modules.archives.service.ProjectManageBusinessOtherService;
import org.jeecg.modules.payment.service.ManagerPaymentAndReceiptSlipService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
   * @Author jihaosen
   * @date 2021/2/24
   */
@Api(value = "项目档案接口")
@RestController
@RequestMapping("/archives/projectManageArchives")
@Slf4j
public class ProjectManageArchivesController {
	@Autowired
	private ProjectManageArchivesService projectManageArchivesService;
	@Autowired
	private ManagerPaymentAndReceiptSlipService managerPaymentAndReceiptSlipService;
	@Autowired
	private ProjectManageBusinessOtherService projectManageBusinessOtherService;
	@Autowired
	private ProjectManageArchivesAndBusinessService projectManageArchivesAndBusinessService;


	@ApiOperation(value = "项目档案分页查询接口", notes = "项目档案分页查询")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "pkOrg", value = "组织", required = true, dataType = "String")
	})
	@GetMapping(value = "/list")
	public Result<IPage<ProjectManageArchives>> queryPageList(ProjectManageArchives projectManageArchives,
			  @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
			  @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
			  HttpServletRequest req) {
		Result<IPage<ProjectManageArchives>> result = new Result<IPage<ProjectManageArchives>>();
		QueryWrapper<ProjectManageArchives> queryWrapper = QueryGenerator.initQueryWrapper(projectManageArchives, req.getParameterMap());
		Page<ProjectManageArchives> page = new Page<ProjectManageArchives>(pageNo, pageSize);
		queryWrapper.eq("del_flag","0").orderByDesc("create_time");
		IPage<ProjectManageArchives> pageList = projectManageArchivesService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}
	
	@ApiOperation(value = "新增项目档案接口", notes = "新增项目档案")
	@ApiImplicitParams({
			@ApiImplicitParam(name="proId", value="项目档案Id",required=true, dataType="String"),
			@ApiImplicitParam(name="proCode", value="项目编码 （项目档案）",required=true, dataType="String"),
			@ApiImplicitParam(name="proName", value="项目名称（项目档案）",required=true, dataType="String"),
			@ApiImplicitParam(name="cusId", value="客户档案id",required=true, dataType="String"),
			@ApiImplicitParam(name="cusCode", value="客户编码 (客户档案)",required=true, dataType="String"),
			@ApiImplicitParam(name="cusName", value="客户名称 （客户档案）",required=true, dataType="String"),
			@ApiImplicitParam(name="totalPrice", value="项目总金额",required=true, dataType="String"),
			@ApiImplicitParam(name="saleManagerId", value="销售经理id(角色档案)",required=false, dataType="String"),
			@ApiImplicitParam(name="saleManager", value="销售经理",required=false, dataType="String"),
			@ApiImplicitParam(name="saleManId", value="业务员id(角色档案)",required=false, dataType="String"),
			@ApiImplicitParam(name="saleMan", value="业务员",required=false, dataType="String"),
			@ApiImplicitParam(name="impManagerId", value="实施项目经理id(角色档案)",required=false, dataType="String"),
			@ApiImplicitParam(name="impManager", value="实施项目经理",required=false, dataType="String"),
			@ApiImplicitParam(name="impConsultantId", value="实施顾问id(角色档案)",required=false, dataType="String"),
			@ApiImplicitParam(name="impConsultant", value="实施顾问",required=false, dataType="String"),
			@ApiImplicitParam(name="deManagerId", value="开发项目经理id(角色档案)",required=false, dataType="String"),
			@ApiImplicitParam(name="deManager", value="开发项目经理",required=false, dataType="String"),
			@ApiImplicitParam(name="deEngineerId", value="开发工程师id(角色档案)",required=false, dataType="String"),
			@ApiImplicitParam(name="deEngineer", value="开发工程师",required=false, dataType="String"),
			@ApiImplicitParam(name="seManagerId", value="服务经理id(角色档案)",required=true, dataType="String"),
			@ApiImplicitParam(name="seManager", value="服务经理",required=true, dataType="String"),
			@ApiImplicitParam(name="seEngineerId", value="服务工程师id",required=true, dataType="String"),
			@ApiImplicitParam(name="seEngineer", value="服务工程师",required=true, dataType="String"),
			@ApiImplicitParam(name="pkOrg", value="组织",required=false, dataType="String"),
			@ApiImplicitParam(name="planId", value="计划id",required=false, dataType="String"),
			@ApiImplicitParam(name="planName", value="计划名称",required=false, dataType="String"),
			@ApiImplicitParam(name="price1", value="第一笔",required=false, dataType="String"),
			@ApiImplicitParam(name="price2", value="第二笔",required=false, dataType="String"),
			@ApiImplicitParam(name="price3", value="第三笔",required=false, dataType="String"),
			@ApiImplicitParam(name="price4", value="第四笔",required=false, dataType="String"),
			@ApiImplicitParam(name="price5", value="第五笔",required=false, dataType="String"),
			@ApiImplicitParam(name="milestone1", value="对应里程碑1",required=false, dataType="String"),
			@ApiImplicitParam(name="milestone2", value="对应里程碑2",required=false, dataType="String"),
			@ApiImplicitParam(name="milestone3", value="对应里程碑3",required=false, dataType="String"),
			@ApiImplicitParam(name="milestone4", value="对应里程碑4",required=false, dataType="String"),
			@ApiImplicitParam(name="milestone5", value="对应里程碑5",required=false, dataType="String"),
			@ApiImplicitParam(name="proArchivesId", value="项目档案id",required=false, dataType="String"),
			@ApiImplicitParam(name="key", value="当前节点id",required=false, dataType="String"),
			@ApiImplicitParam(name="parentId", value="父节点id",required=false, dataType="String"),
			@ApiImplicitParam(name="muilesId", value="里程碑id",required=false, dataType="String"),
			@ApiImplicitParam(name="muilesName", value="里程碑名称",required=false, dataType="String"),
			@ApiImplicitParam(name="startDate", value="开始时间",required=false, dataType="String"),
			@ApiImplicitParam(name="endDate", value="结束时间",required=false, dataType="String"),
			@ApiImplicitParam(name="planTime", value="计划人天",required=false, dataType="String"),
			@ApiImplicitParam(name="executor", value="执行人",required=false, dataType="String"),
			@ApiImplicitParam(name="executorId", value="执行人id",required=false, dataType="String"),
			@ApiImplicitParam(name="realTime", value="实际人天",required=false, dataType="String"),
	})
	@PostMapping(value = "/add")
	@NotDuplicate
	public Result<ProjectManageArchives> add(@RequestBody ProArchivesAddReqDTO reqDTO, BindingResult bindingResult) {
		Result<ProjectManageArchives> result = new Result<ProjectManageArchives>();
		try {
			StringBuilder sb = new StringBuilder();
			if (bindingResult.hasErrors()){
				//记录错误信息
				bindingResult.getAllErrors().stream().forEach(error -> sb.append(error.getDefaultMessage() + "<br/>"));
				result.error500(sb.toString());
				return result;
			}
			projectManageArchivesService.add(reqDTO);
			result.success("添加成功！");
		} catch (Exception e) {
			e.printStackTrace();
			log.info(e.getMessage());
			result.error500("操作失败");
		}
		return result;
	}

	@ApiOperation(value = "编辑项目档案接口", notes = "编辑项目档案")
	@ApiImplicitParams({
			@ApiImplicitParam(name="id", value="id",required=true, dataType="String"),
			@ApiImplicitParam(name="proId", value="项目档案Id",required=true, dataType="String"),
			@ApiImplicitParam(name="proCode", value="项目编码 （项目档案）",required=true, dataType="String"),
			@ApiImplicitParam(name="proName", value="项目名称（项目档案）",required=true, dataType="String"),
			@ApiImplicitParam(name="cusId", value="客户档案id",required=true, dataType="String"),
			@ApiImplicitParam(name="cusCode", value="客户编码 (客户档案)",required=true, dataType="String"),
			@ApiImplicitParam(name="cusName", value="客户名称 （客户档案）",required=true, dataType="String"),
			@ApiImplicitParam(name="totalPrice", value="项目总金额",required=true, dataType="String"),
			@ApiImplicitParam(name="saleManagerId", value="销售经理id(角色档案)",required=false, dataType="String"),
			@ApiImplicitParam(name="saleManager", value="销售经理",required=false, dataType="String"),
			@ApiImplicitParam(name="saleManId", value="业务员id(角色档案)",required=false, dataType="String"),
			@ApiImplicitParam(name="saleMan", value="业务员",required=false, dataType="String"),
			@ApiImplicitParam(name="impManagerId", value="实施项目经理id(角色档案)",required=false, dataType="String"),
			@ApiImplicitParam(name="impManager", value="实施项目经理",required=false, dataType="String"),
			@ApiImplicitParam(name="impConsultantId", value="实施顾问id(角色档案)",required=false, dataType="String"),
			@ApiImplicitParam(name="impConsultant", value="实施顾问",required=false, dataType="String"),
			@ApiImplicitParam(name="deManagerId", value="开发项目经理id(角色档案)",required=false, dataType="String"),
			@ApiImplicitParam(name="deManager", value="开发项目经理",required=false, dataType="String"),
			@ApiImplicitParam(name="deEngineerId", value="开发工程师id(角色档案)",required=false, dataType="String"),
			@ApiImplicitParam(name="deEngineer", value="开发工程师",required=false, dataType="String"),
			@ApiImplicitParam(name="seManagerId", value="服务经理id(角色档案)",required=true, dataType="String"),
			@ApiImplicitParam(name="seManager", value="服务经理",required=true, dataType="String"),
			@ApiImplicitParam(name="seEngineerId", value="服务工程师id",required=true, dataType="String"),
			@ApiImplicitParam(name="seEngineer", value="服务工程师",required=true, dataType="String"),
			@ApiImplicitParam(name="pkOrg", value="组织",required=false, dataType="String"),
			@ApiImplicitParam(name="planId", value="计划id",required=false, dataType="String"),
			@ApiImplicitParam(name="planName", value="计划名称",required=false, dataType="String"),
			@ApiImplicitParam(name="price1", value="第一笔",required=false, dataType="String"),
			@ApiImplicitParam(name="price2", value="第二笔",required=false, dataType="String"),
			@ApiImplicitParam(name="price3", value="第三笔",required=false, dataType="String"),
			@ApiImplicitParam(name="price4", value="第四笔",required=false, dataType="String"),
			@ApiImplicitParam(name="price5", value="第五笔",required=false, dataType="String"),
			@ApiImplicitParam(name="milestone1", value="对应里程碑1",required=false, dataType="String"),
			@ApiImplicitParam(name="milestone2", value="对应里程碑2",required=false, dataType="String"),
			@ApiImplicitParam(name="milestone3", value="对应里程碑3",required=false, dataType="String"),
			@ApiImplicitParam(name="milestone4", value="对应里程碑4",required=false, dataType="String"),
			@ApiImplicitParam(name="milestone5", value="对应里程碑5",required=false, dataType="String"),
			@ApiImplicitParam(name="proArchivesId", value="项目档案id",required=false, dataType="String"),
			@ApiImplicitParam(name="key", value="当前节点id",required=false, dataType="String"),
			@ApiImplicitParam(name="parentId", value="父节点id",required=false, dataType="String"),
			@ApiImplicitParam(name="muilesId", value="里程碑id",required=false, dataType="String"),
			@ApiImplicitParam(name="muilesName", value="里程碑名称",required=false, dataType="String"),
			@ApiImplicitParam(name="startDate", value="开始时间",required=false, dataType="String"),
			@ApiImplicitParam(name="endDate", value="结束时间",required=false, dataType="String"),
			@ApiImplicitParam(name="planTime", value="计划人天",required=false, dataType="String"),
			@ApiImplicitParam(name="executor", value="执行人",required=false, dataType="String"),
			@ApiImplicitParam(name="executorId", value="执行人id",required=false, dataType="String"),
			@ApiImplicitParam(name="realTime", value="实际人天",required=false, dataType="String"),
	})
	@PostMapping(value = "/edit")
	public Result<ProjectManageArchives> edit(@RequestBody ProArchivesAddReqDTO reqDTO) {
		Result<ProjectManageArchives> result = new Result<ProjectManageArchives>();
		ProjectManageArchives projectManageArchivesEntity = projectManageArchivesService.getById(reqDTO.getId());
		if(projectManageArchivesEntity==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = projectManageArchivesService.edit(reqDTO);
			if(ok) {
				result.success("修改成功!");
			}
		}
		
		return result;
	}
	
	@ApiOperation(value = "项目档案通过id删除接口", notes = "通过id删除项目档案")
	@ApiImplicitParams({
			@ApiImplicitParam(name="id", value="id",required=true, dataType="String")
	})
	@PostMapping(value = "/delete")
	public Result<ProjectManageArchives> delete(@RequestBody ProjectManageArchives archives) {
		Result<ProjectManageArchives> result = new Result<ProjectManageArchives>();
		ProjectManageArchives projectManageArchives = projectManageArchivesService.getById(archives.getId());
		if(projectManageArchives==null) {
			result.error500("未找到对应实体");
		}else {
			projectManageArchives.setDelFlag("1");
			boolean ok = projectManageArchivesService.deleteById(projectManageArchives);
			if(ok) {
				result.success("删除成功!");
			}
		}
		return result;
	}
	
	@ApiOperation(value = "项目档案通过id查询接口", notes = "根据id查询项目档案")
	@ApiImplicitParams({
			@ApiImplicitParam(name="id", value="id",required=true, dataType="String")
	})
	@GetMapping(value = "/queryById")
	public Result<ProjectManageArchives> queryById(@RequestParam(name="id",required=true) String id) {
		Result<ProjectManageArchives> result = new Result<ProjectManageArchives>();
		ProjectManageArchives projectManageArchives = projectManageArchivesService.getById(id);
		if(projectManageArchives==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(projectManageArchives);
			result.setSuccess(true);
		}
		return result;
	}

	@ApiOperation(value = "项目档案主子表查询接口", notes = "根据id查询项目档案主子表")
	@ApiImplicitParams({
			@ApiImplicitParam(name="id", value="id",required=true, dataType="String")
	})
	@GetMapping(value = "/getListById")
	public Result<ProArchivesAddRespDTO> getListById(@RequestParam(name="id",required=true) String id) {
		Result<ProArchivesAddRespDTO> result = new Result<ProArchivesAddRespDTO>();
		ProjectManageArchives projectManageArchives = projectManageArchivesService.getById(id);
		if(projectManageArchives==null) {
			result.error500("未找到对应实体");
		}else {
			ProArchivesAddRespDTO respDTO = projectManageArchivesService.getListById(projectManageArchives);
			result.setResult(respDTO);
			result.setSuccess(true);
		}
		return result;
	}

	@ApiOperation(value = "统计汇款单金额", notes = "统计汇款单金额")
	@ApiImplicitParams({
			@ApiImplicitParam(name="proId", value="项目id",required=true, dataType="String"),
			@ApiImplicitParam(name="type", value="1.收款单 2.付款单",required=true, dataType="String"),
			@ApiImplicitParam(name="proArchivesId", value="项目档案里程碑id",required=true, dataType="String")
	})
	@GetMapping(value = "/getExpensePrice")
	public Result<ExpensePriceRespDTO> getExpensePrice(@Valid ExpensePriceReqDTO reqDTO, BindingResult bindingResult) {
		Result<ExpensePriceRespDTO> result = new Result<ExpensePriceRespDTO>();
		try {
			StringBuilder sb = new StringBuilder();
			if (bindingResult.hasErrors()){
				//记录错误信息
				bindingResult.getAllErrors().stream().forEach(error -> sb.append(error.getDefaultMessage() + "<br/>"));
				result.error500(sb.toString());
				return result;
			}
		ExpensePriceRespDTO respDTO = managerPaymentAndReceiptSlipService.getExpensePrice(reqDTO);
		result.setResult(respDTO);
		result.setSuccess(true);
		} catch (Exception e) {
			e.printStackTrace();
			log.info(e.getMessage());
			result.error500("操作失败");
		}
		return result;
	}

	@ApiOperation(value = "项目档案计划列表查询", notes = "根据主表id查询项目档案计划列表")
	@ApiImplicitParams({
			@ApiImplicitParam(name="id", value="id",required=true, dataType="String"),
			@ApiImplicitParam(name="planType", value="计划类型",required=true, dataType="String")
	})
	@GetMapping(value = "/getPlanListById")
	public Result<List<ProPlanListRespDTO>> getListById(@Valid ProPlanListReqDTO reqDTO, BindingResult bindingResult) {
		Result<List<ProPlanListRespDTO>> result = new Result<List<ProPlanListRespDTO>>();
		try {
			StringBuilder sb = new StringBuilder();
			if(bindingResult.hasErrors()) {
				//记录错误信息
				bindingResult.getAllErrors().stream().forEach(error -> sb.append(error.getDefaultMessage() + "<br/>"));
				result.error500(sb.toString());
				return result;
			}
			QueryWrapper<ProjectManageBusinessOther> queryWrapper = new QueryWrapper<>();
			queryWrapper.eq("del_flag", "0").eq("pro_archives_id", reqDTO.getId())
					.eq("plan_type", reqDTO.getPlanType());
			queryWrapper.orderByAsc("sort");

			   List<ProjectManageBusinessOther> list = projectManageBusinessOtherService.list(queryWrapper);
			   List<ProPlanListRespDTO> respDTOS = getTreeList(list);
				result.setResult(respDTOS);
				result.setSuccess(true);
		}
		catch (Exception e) {
				e.printStackTrace();
				log.info(e.getMessage());
				result.error500("操作失败");
			}
		return result;
	}

	@ApiOperation(value = "项目档案—商务收款计划", notes = "根据主表id查询项目档案计划列表")
	@ApiImplicitParams({
			@ApiImplicitParam(name="id", value="项目档案id",required=true, dataType="String"),
			@ApiImplicitParam(name="planType", value="计划类型",required=true, dataType="String"),
			@ApiImplicitParam(name="pkOrg", value="组织",required=true, dataType="String")
	})
	@GetMapping(value = "/getBusinessList")
	public Result<List<ArchivesBusinessListRespDTO>> getBusinessList(@Valid ArchivesBusinessListReqDTO reqDTO, BindingResult bindingResult) {
		Result<List<ArchivesBusinessListRespDTO>> result = new Result<List<ArchivesBusinessListRespDTO>>();
		try {
			StringBuilder sb = new StringBuilder();
			if(bindingResult.hasErrors()) {
				//记录错误信息
				bindingResult.getAllErrors().stream().forEach(error -> sb.append(error.getDefaultMessage() + "<br/>"));
				result.error500(sb.toString());
				return result;
			}
			QueryWrapper<ProjectManageArchivesAndBusiness> queryWrapper = new QueryWrapper<>();
			queryWrapper.eq("del_flag", "0")
					.eq("pro_archives_id", reqDTO.getId());
			if(StringUtils.isNotBlank(reqDTO.getPlanType())){
				queryWrapper.eq("plan_type", reqDTO.getPlanType());
			}
			if(StringUtils.isNotBlank(reqDTO.getPkOrg())){
				queryWrapper.eq("pkOrg", reqDTO.getPkOrg());
			}
			if(StringUtils.isNotBlank(reqDTO.getPlanId())){
				queryWrapper.eq("plan_id", reqDTO.getPlanId());
			}
			    queryWrapper.orderByAsc("sort");
			    List<ProjectManageArchivesAndBusiness> list = projectManageArchivesAndBusinessService.list(queryWrapper);
			    List<ArchivesBusinessListRespDTO> respDTOS = Lists.newArrayList();
                if(!CollectionUtils.isEmpty(list)){
					ProjectManageArchivesAndBusiness business = list.get(0);
					if(StringUtils.isNotBlank(business.getMileId1())){
						respDTOS.add(new ArchivesBusinessListRespDTO(business.getId(), business.getMileId1(), business.getMilestone1(), business.getPrice1()));
					}
					if(StringUtils.isNotBlank(business.getMileId2())){
						respDTOS.add(new ArchivesBusinessListRespDTO(business.getId(), business.getMileId2(), business.getMilestone2(), business.getPrice2()));
					}
					if(StringUtils.isNotBlank(business.getMileId3())){
						respDTOS.add(new ArchivesBusinessListRespDTO(business.getId(), business.getMileId3(), business.getMilestone3(), business.getPrice3()));
					}
					if(StringUtils.isNotBlank(business.getMileId4())){
						respDTOS.add(new ArchivesBusinessListRespDTO(business.getId(), business.getMileId4(), business.getMilestone4(), business.getPrice4()));

					}
					if(StringUtils.isNotBlank(business.getMileId5())){
						respDTOS.add(new ArchivesBusinessListRespDTO(business.getId(), business.getMileId5(), business.getMilestone5(), business.getPrice5()));
					}
				}


				result.setResult(respDTOS);
				result.setSuccess(true);
		}
		catch (Exception e) {
				e.printStackTrace();
				log.info(e.getMessage());
				result.error500("操作失败");
			}
		return result;
	}



	/**
	 * @desc 将计划列表转成树状结构
	 */
	private List<ProPlanListRespDTO> getTreeList(List<ProjectManageBusinessOther> list) {
		List<ProPlanListRespDTO> resp = Lists.newArrayList();
		//根据parentId分组
		Map<String, List<ProjectManageBusinessOther>> postMap = list.stream().collect(Collectors.groupingBy(ProjectManageBusinessOther :: getParentId));
		List<ProjectManageBusinessOther>  baseArchivesMilestones = postMap.get("0");
		if(CollectionUtils.isEmpty(baseArchivesMilestones)){
			return null;
		}
		for(ProjectManageBusinessOther other : baseArchivesMilestones){
			ProPlanListRespDTO respDTO = new ProPlanListRespDTO(other);
			respDTO.setChildren(getChildes(list, respDTO.getKey(), postMap));
			//判断是否为末节点
			List<ProjectManageBusinessOther> posts = postMap.get(respDTO.getKey());
			if(CollectionUtils.isEmpty(posts)){
				respDTO.setIsLeaf(true);
				respDTO.setChildren(null);
			}
			resp.add(respDTO);
		}
		return resp;
	}

	/**
	 * @desc 获取子节点数据
	 */
	private List<ProPlanListRespDTO> getChildes(List<ProjectManageBusinessOther> list, String key, Map<String, List<ProjectManageBusinessOther>> postMap) {
		List<ProPlanListRespDTO> resp = Lists.newArrayList();
		for(ProjectManageBusinessOther other : list){
			if(other.getParentId().equals(key)){
				ProPlanListRespDTO respDTO = new ProPlanListRespDTO(other);
				respDTO.setChildren(getChildes(list, respDTO.getKey(), postMap));
				//判断是否为末节点
				List<ProjectManageBusinessOther> posts = postMap.get(respDTO.getKey());
				if(CollectionUtils.isEmpty(posts)){
					respDTO.setIsLeaf(true);
					respDTO.setChildren(null);
				}
				resp.add(respDTO);
			}
		}
		return resp;
	}

}
