package org.jeecg.modules.archives.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.google.common.collect.Lists;
import org.apache.commons.lang.ArrayUtils;
import org.apache.commons.lang.StringUtils;
import org.jeecg.common.dto.archives.BsInfo;
import org.jeecg.common.dto.archives.DeInfo;
import org.jeecg.common.dto.archives.DeInfoRespDTO;
import org.jeecg.common.dto.archives.IpInfo;
import org.jeecg.common.dto.archives.IpInfoRespDTO;
import org.jeecg.common.dto.archives.ProArchivesAddReqDTO;
import org.jeecg.common.dto.archives.ProArchivesAddRespDTO;
import org.jeecg.common.dto.archives.ProArchivesAndBusinessAddReqDTO;
import org.jeecg.common.dto.archives.ProBusinessOtherAddReqDTO;
import org.jeecg.common.dto.archives.ProBusinessOtherListRespDTO;
import org.jeecg.common.dto.archives.SeInfo;
import org.jeecg.common.dto.archives.SeInfoRespDTO;
import org.jeecg.common.exception.JeecgBootException;
import org.jeecg.modules.archives.entity.ProjectManageArchives;
import org.jeecg.modules.archives.entity.ProjectManageArchivesAndBusiness;
import org.jeecg.modules.archives.entity.ProjectManageBusinessOther;
import org.jeecg.modules.archives.mapper.ProjectManageArchivesMapper;
import org.jeecg.modules.archives.service.ProjectManageArchivesAndBusinessService;
import org.jeecg.modules.archives.service.ProjectManageArchivesService;
import org.jeecg.modules.archives.service.ProjectManageBusinessOtherService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

 /** 
   * @Author jihaosen
   * @date 2021/2/2
   * @desc 项目档案 service 服务
   */
 
@Service
@Transactional
public class ProjectManageArchivesServiceImpl extends ServiceImpl<ProjectManageArchivesMapper, ProjectManageArchives> implements ProjectManageArchivesService {
    @Autowired
    private ProjectManageArchivesAndBusinessService projectManageArchivesAndBusinessService;
    @Autowired
    private ProjectManageBusinessOtherService projectManageBusinessOtherService;

    /**
     * @desc 新增
     */
    @Override
    public void add(ProArchivesAddReqDTO reqDTO) throws Exception{
        if(reqDTO == null){
            throw new JeecgBootException("项目档案不能为空");
        }
        //保存项目档案数据
        ProjectManageArchives projectManageArchives = new ProjectManageArchives(reqDTO);
        save(projectManageArchives);

        //保存子表数据
        reqDTO.setId(projectManageArchives.getId());
        saveArchivesAndBusiness(reqDTO);

    }

    /**
     * @desc 编辑
     */
    @Override
    public boolean edit(ProArchivesAddReqDTO reqDTO) {
        //修改项目档案数据
        ProjectManageArchives projectManageArchives = new ProjectManageArchives(reqDTO);
        projectManageArchives.setId(reqDTO.getId());
        updateById(projectManageArchives);

        //逻辑删除 相关子表
        deleteArchivesAndBusiness(reqDTO.getId());

        //保存子表数据
        saveArchivesAndBusiness(reqDTO);
        return true;
    }

    /**
     * @desc 根据id查询项目档案主子表
     */
    @Override
    public ProArchivesAddRespDTO getListById(ProjectManageArchives projectManageArchives) {
        ProArchivesAddRespDTO respDTO = new ProArchivesAddRespDTO(projectManageArchives);
        QueryWrapper<ProjectManageArchivesAndBusiness> queryWrapper = new QueryWrapper();
        queryWrapper.eq("del_flag","0");
        queryWrapper.eq("pro_archives_id",projectManageArchives.getId());
        queryWrapper.orderByAsc("create_time");
        List<ProjectManageArchivesAndBusiness>  businessList = projectManageArchivesAndBusinessService.list(queryWrapper);

        QueryWrapper<ProjectManageBusinessOther> otherQueryWrapper = new QueryWrapper();
        otherQueryWrapper.eq("del_flag","0");
        otherQueryWrapper.eq("pro_archives_id",projectManageArchives.getId());
        otherQueryWrapper.orderByAsc("create_time");
        List<ProjectManageBusinessOther>  otherList = projectManageBusinessOtherService.list(otherQueryWrapper);
        Map<Integer, List<ProjectManageBusinessOther>> otherMap = otherList.stream().collect(Collectors.groupingBy(ProjectManageBusinessOther :: getPlanType));

        setRespDTO(respDTO, businessList, otherMap);

        return  respDTO;
    }

    /**
     * @desc 通过id删除项目档案
     */
    @Override
    public boolean deleteById(ProjectManageArchives projectManageArchives) {
        updateById(projectManageArchives);
        //逻辑删除 相关子表
        deleteArchivesAndBusiness(projectManageArchives.getId());
        return true;
    }



    /**
     * @desc 设置里程碑数据
     */
    private void setRespDTO(ProArchivesAddRespDTO respDTO, List<ProjectManageArchivesAndBusiness> businessList, Map<Integer, List<ProjectManageBusinessOther>> otherMap) {
        respDTO.setBsInfo(new BsInfo(getBsInfoData(businessList, 1), getBsInfoData(businessList, 2),
                getBsInfoData(businessList, 3), getBsInfoData(businessList, 4)));
        respDTO.setIpInfo(new IpInfoRespDTO(getPlanList(otherMap.get(2))));
        respDTO.setDeInfo(new DeInfoRespDTO(getPlanList(otherMap.get(3))));
        respDTO.setSeInfo(new SeInfoRespDTO(getPlanList(otherMap.get(4))));
    }

    /**
     * @desc 将里程碑计划数据转成树状结构
     */
    private List<ProBusinessOtherListRespDTO> getPlanList(List<ProjectManageBusinessOther> otherList) {
        if(!CollectionUtils.isEmpty(otherList)){
            otherList.sort((x, y) -> Integer.compare(x.getSort(), y.getSort()));
        }

        List<ProBusinessOtherListRespDTO> resp = Lists.newArrayList();
        //根据parentId分组
        Map<String, List<ProjectManageBusinessOther>> postMap = otherList.stream().collect(Collectors.groupingBy(ProjectManageBusinessOther :: getParentId));
        List<ProjectManageBusinessOther>  baseArchivesMilestones = postMap.get("0");
        if(CollectionUtils.isEmpty(baseArchivesMilestones)){
            return null;
        }
        for(ProjectManageBusinessOther other : baseArchivesMilestones){
            ProBusinessOtherListRespDTO respDTO = new ProBusinessOtherListRespDTO(other, getExecutorList(other));
            respDTO.setChildren(getChildes(otherList, respDTO.getKey(), postMap));
            //判断是否为末节点
            List<ProjectManageBusinessOther> posts = postMap.get(respDTO.getKey());
            if(CollectionUtils.isEmpty(posts)){
                respDTO.setIsLeaf(true);
                respDTO.setChildren(null);
            }
            resp.add(respDTO);
        }
        return resp;
    }

     /**
      * @desc 获取执行人列表
      */
     private List<String> getExecutorList(ProjectManageBusinessOther other) {
         return StringUtils.isNotBlank(other.getExecutorId()) ? Arrays.asList(other.getExecutorId().split("、")) : null;
     }

     /**
     * @desc 获取子节点数据
     */
    private List<ProBusinessOtherListRespDTO> getChildes(List<ProjectManageBusinessOther> list, String key, Map<String, List<ProjectManageBusinessOther>> postMap) {
        List<ProBusinessOtherListRespDTO> resp = Lists.newArrayList();
        for(ProjectManageBusinessOther other : list){
            if(other.getParentId().equals(key)){
                ProBusinessOtherListRespDTO respDTO = new ProBusinessOtherListRespDTO(other, getExecutorList(other));
                respDTO.setChildren(getChildes(list, respDTO.getKey(), postMap));
                //判断是否为末节点
                List<ProjectManageBusinessOther> posts = postMap.get(respDTO.getKey());
                if(CollectionUtils.isEmpty(posts)){
                    respDTO.setIsLeaf(true);
                    respDTO.setChildren(null);
                }
                resp.add(respDTO);
            }
        }
        return resp;
    }

    /**
     * @desc 获取里程碑数据
     */
    private List<ProArchivesAndBusinessAddReqDTO> getBsInfoData(List<ProjectManageArchivesAndBusiness> businessList, int planType) {
        if(!CollectionUtils.isEmpty(businessList)){
            businessList.sort((x, y) -> Integer.compare(x.getSort(), y.getSort()));
        }
        List<ProArchivesAndBusinessAddReqDTO> reqDTOS = Lists.newArrayList();
        for(ProjectManageArchivesAndBusiness business : businessList){
           int currentPlanType = business.getPlanType();
            if(planType == currentPlanType){
                reqDTOS.add(new ProArchivesAndBusinessAddReqDTO(business));
            }
        }
        return reqDTOS;
    }

    /**
     * @desc 逻辑删除 子表
     */
    private void deleteArchivesAndBusiness(String id) {
        //删除项目档案--里程碑信息表
        QueryWrapper<ProjectManageArchivesAndBusiness> businessQueryWrapper = new QueryWrapper<>();
        businessQueryWrapper.eq("del_flag", 0).eq("pro_archives_id", id);
        List<ProjectManageArchivesAndBusiness> businessList = projectManageArchivesAndBusinessService.list(businessQueryWrapper);
        if(!CollectionUtils.isEmpty(businessList)){
            projectManageArchivesAndBusinessService.deleteBatch(businessList);
        }
        //删除项目档案--计划表
        QueryWrapper<ProjectManageBusinessOther> archivesAndBusinessQueryWrapper = new QueryWrapper<>();
        archivesAndBusinessQueryWrapper.eq("del_flag", 0).eq("pro_archives_id", id);
        List<ProjectManageBusinessOther>  businessOtherList = projectManageBusinessOtherService.list(archivesAndBusinessQueryWrapper);
        if(!CollectionUtils.isEmpty(businessOtherList)){
            projectManageBusinessOtherService.deleteBatch(businessOtherList);
        }
    }

    /**
     * @desc 添加项目档案--商务,实施,开发,服务的数据
     */
    private void saveArchivesAndBusiness(ProArchivesAddReqDTO reqDTO) {
        //添加 项目档案—商务
        saveBsInfo(reqDTO);
        //添加 项目档案—实施
        saveIpInfo(reqDTO);
        //添加 项目档案—开发
        saveDeInfo(reqDTO);
        //添加 项目档案—服务
        saveSeInfo(reqDTO);
    }

    //添加 项目档案—商务
    private void saveBsInfo(ProArchivesAddReqDTO reqDTO){
        BsInfo bsInfo = reqDTO.getBsInfo();
        if(bsInfo == null){
            throw new JeecgBootException("项目档案—商务的数据不能为空");
        }
        //保存收款计划
        List<ProArchivesAndBusinessAddReqDTO> coPlanList = bsInfo.getCoPlanList();
        if(!CollectionUtils.isEmpty(coPlanList)){
            saveMileData( coPlanList,  reqDTO);
        }
        //保存收款情况
        List<ProArchivesAndBusinessAddReqDTO> coSituationList = bsInfo.getCoSituationList();
        if(!CollectionUtils.isEmpty(coSituationList)){
            saveMileData(coSituationList,  reqDTO);
        }
        //保存付款计划
        List<ProArchivesAndBusinessAddReqDTO> payPlanList = bsInfo.getPayPlanList();
        if(!CollectionUtils.isEmpty(payPlanList)){
            saveMileData(payPlanList,  reqDTO);
        }

        //保存付款情况
        List<ProArchivesAndBusinessAddReqDTO> paySituationList = bsInfo.getPaySituationList();
        if(!CollectionUtils.isEmpty(paySituationList)){
            saveMileData(paySituationList,  reqDTO);
        }
    }

    /**
     * @desc 添加 项目档案—实施
     */
    private void saveIpInfo(ProArchivesAddReqDTO reqDTO) {
        IpInfo ipInfo =reqDTO.getIpInfo();
        if(ipInfo == null){
            throw new JeecgBootException("项目档案—实施的数据不能为空");
        }
//        List<ProArchivesAndBusinessAddReqDTO> mileList = ipInfo.getMileList();
//        if(CollectionUtils.isEmpty(mileList)){
//            throw new JeecgBootException("项目档案—实施的里程碑数据不能为空");
//        }
        //保存里程碑数据
//        saveMileData(mileList, reqDTO, 2);

        List<ProBusinessOtherAddReqDTO> planList = ipInfo.getPlanList();
        if(CollectionUtils.isEmpty(planList)){
            throw new JeecgBootException("项目档案—实施的里程碑计划数据不能为空");
        }
        //保存里程碑计划数据
        saveMolePlanData(planList, reqDTO, 2);

    }

     /** 
       * @desc 添加 项目档案—开发
       */
    private void saveDeInfo(ProArchivesAddReqDTO reqDTO) {
        DeInfo deInfo =reqDTO.getDeInfo();
        if(deInfo == null){
            throw new JeecgBootException("项目档案—开发的数据不能为空");
        }
//        List<ProArchivesAndBusinessAddReqDTO> mileList = deInfo.getMileList();
//        if(CollectionUtils.isEmpty(mileList)){
//            throw new JeecgBootException("项目档案—开发的里程碑数据不能为空");
//        }
       //保存里程碑数据
//        saveMileData(mileList, reqDTO, 3);

        List<ProBusinessOtherAddReqDTO> planList = deInfo.getPlanList();
        if(CollectionUtils.isEmpty(planList)){
            throw new JeecgBootException("项目档案—开发的里程碑计划数据不能为空");
        }
        //保存里程碑计划数据
        saveMolePlanData(planList, reqDTO, 3);
    }

    /**
     * @desc 添加 项目档案—服务
     */
    private void saveSeInfo(ProArchivesAddReqDTO reqDTO) {
        SeInfo seInfo =reqDTO.getSeInfo();
        if(seInfo == null){
            throw new JeecgBootException("项目档案—服务的数据不能为空");
        }
//        List<ProArchivesAndBusinessAddReqDTO> mileList = seInfo.getMileList();
//        if(CollectionUtils.isEmpty(mileList)){
//            throw new JeecgBootException("项目档案—服务的里程碑数据不能为空");
//        }
//        //保存里程碑数据
//        saveMileData(mileList, reqDTO, 4);

        List<ProBusinessOtherAddReqDTO> planList = seInfo.getPlanList();
        if(CollectionUtils.isEmpty(planList)){
            throw new JeecgBootException("项目档案—服务的里程碑计划数据不能为空");
        }
        //保存里程碑计划数据
        saveMolePlanData(planList, reqDTO, 4);
    }
    /**
     * @desc 保存里程碑数据
     */
    private void saveMileData(List<ProArchivesAndBusinessAddReqDTO> mileList, ProArchivesAddReqDTO reqDTO) {
        List<ProjectManageArchivesAndBusiness> list = Lists.newArrayList();
        int sort = 1;
        for(ProArchivesAndBusinessAddReqDTO mile :  mileList) {
            ProjectManageArchivesAndBusiness business = new ProjectManageArchivesAndBusiness(mile, reqDTO.getId(), reqDTO.getPkOrg());
            business.setSort(sort++);
            list.add(business);
        }
        projectManageArchivesAndBusinessService.saveBatch(list);
    }

    /**
     * @desc 保存里程碑计划数据
     */
    private void saveMolePlanData(List<ProBusinessOtherAddReqDTO> planList, ProArchivesAddReqDTO reqDTO, Integer planType) {
        List<ProjectManageBusinessOther> list = Lists.newArrayList();
        int sort = 1;
        for(ProBusinessOtherAddReqDTO plan :  planList) {
            ProjectManageBusinessOther other = new ProjectManageBusinessOther(plan, reqDTO.getId(), planType, reqDTO.getPkOrg());
            other.setSort(sort++);
            list.add(other);
        }
        projectManageBusinessOtherService.saveBatch(list);
    }

}
