package org.jeecg.modules.basedata.controller;

import javax.servlet.http.HttpServletRequest;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.constant.CommonConstant;
import org.jeecg.common.dto.basedata.ArchivesCollectionLineReqDTO;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.modules.basedata.entity.BaseArchivesCollectionLine;
import org.jeecg.modules.basedata.service.IBaseArchivesCollectionLineService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

/**
   * @Author jihaosen
   * @date 2021/3/2
   */
 @Api("收付款条线档案接口")
@RestController
@RequestMapping("/basedata/baseArchivesCollectionLine")
@Slf4j
public class BaseArchivesCollectionLineController {
	@Autowired
	private IBaseArchivesCollectionLineService baseArchivesCollectionLineService;

	 @ApiOperation(value = "分页查询", notes = "分页查询")
	 @ApiImplicitParams({
			 @ApiImplicitParam(name="pkOrg", value="组织",required=false, dataType="String"),
	 })
	@GetMapping(value = "/list")
	public Result<IPage<BaseArchivesCollectionLine>> queryPageList(BaseArchivesCollectionLine baseArchivesCollectionLine,
	  @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
	  @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
	  HttpServletRequest req) {
		Result<IPage<BaseArchivesCollectionLine>> result = new Result<IPage<BaseArchivesCollectionLine>>();
		baseArchivesCollectionLine.setDelFlag(null);
		QueryWrapper<BaseArchivesCollectionLine> queryWrapper = QueryGenerator.initQueryWrapper(baseArchivesCollectionLine, req.getParameterMap());
		queryWrapper.eq("del_flag", CommonConstant.STATUS_NORMAL.toString());
		 queryWrapper.orderByAsc("create_time");
		Page<BaseArchivesCollectionLine> page = new Page<BaseArchivesCollectionLine>(pageNo, pageSize);
		IPage<BaseArchivesCollectionLine> pageList = baseArchivesCollectionLineService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}

	 @ApiOperation(value = "新增接口", notes = "新增收付款条线档案")
	 @ApiImplicitParams({
			 @ApiImplicitParam(name="pkOrg", value="组织",required=false, dataType="String"),
			 @ApiImplicitParam(name="code", value="编码",required=true, dataType="String"),
			 @ApiImplicitParam(name="name", value="名称",required=true, dataType="String"),
	 })
	@PostMapping(value = "/add")
	public Result<BaseArchivesCollectionLine> add(@RequestBody BaseArchivesCollectionLine baseArchivesCollectionLine,
			BindingResult bindingResult) {
		Result<BaseArchivesCollectionLine> result = new Result<BaseArchivesCollectionLine>();
		try {
			StringBuilder sb = new StringBuilder();
			if (bindingResult.hasErrors()){
				//记录错误信息
				bindingResult.getAllErrors().stream().forEach(error -> sb.append(error.getDefaultMessage() + "<br/>"));
				result.error500(sb.toString());
				return result;
			}
			baseArchivesCollectionLineService.save(baseArchivesCollectionLine);
			result.success("添加成功！");
		} catch (Exception e) {
			e.printStackTrace();
			log.info(e.getMessage());
			result.error500("操作失败");
		}
		return result;
	}

	 @ApiOperation(value = "编辑接口", notes = "编辑收付款条线档案")
	 @ApiImplicitParams({
			 @ApiImplicitParam(name="id", value="id",required=false, dataType="String"),
			 @ApiImplicitParam(name="pkOrg", value="组织",required=false, dataType="String"),
			 @ApiImplicitParam(name="code", value="编码",required=true, dataType="String"),
			 @ApiImplicitParam(name="name", value="名称",required=true, dataType="String"),
	 })
	@PutMapping(value = "/edit")
	public Result<BaseArchivesCollectionLine> edit(@RequestBody BaseArchivesCollectionLine baseArchivesCollectionLine) {
		Result<BaseArchivesCollectionLine> result = new Result<BaseArchivesCollectionLine>();
		BaseArchivesCollectionLine baseArchivesCollectionLineEntity = baseArchivesCollectionLineService.getById(baseArchivesCollectionLine.getId());
		if(baseArchivesCollectionLineEntity==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = baseArchivesCollectionLineService.updateById(baseArchivesCollectionLine);
			if(ok) {
				result.success("修改成功!");
			}
		}
		
		return result;
	}

	 @ApiOperation(value = "删除接口", notes = "删除收付款条线档案")
	 @ApiImplicitParams({
			 @ApiImplicitParam(name="id", value="id",required=false, dataType="String"),
			 @ApiImplicitParam(name="pkOrg", value="组织",required=false, dataType="String"),
			 @ApiImplicitParam(name="code", value="编码",required=true, dataType="String"),
			 @ApiImplicitParam(name="name", value="名称",required=true, dataType="String"),
	 })
	@DeleteMapping(value = "/delete")
	public Result<BaseArchivesCollectionLine> delete(@RequestBody ArchivesCollectionLineReqDTO reqDTO) {
		Result<BaseArchivesCollectionLine> result = new Result<BaseArchivesCollectionLine>();
		BaseArchivesCollectionLine baseArchivesCollectionLine = baseArchivesCollectionLineService.getById(reqDTO.getId());
		if(baseArchivesCollectionLine==null) {
			result.error500("未找到对应实体");
		}else {
			baseArchivesCollectionLine.setDelFlag(CommonConstant.DEL_FLAG_1.toString());
			boolean ok = baseArchivesCollectionLineService.updateById(baseArchivesCollectionLine);
			if(ok) {
				result.success("删除成功!");
			}
		}
		
		return result;
	}


	 @ApiOperation(value = "通过id查询接口", notes = "通过id查询收付款条线档案")
	 @ApiImplicitParams({
			 @ApiImplicitParam(name="id", value="id",required=true, dataType="String"),
	 })
	@GetMapping(value = "/queryById")
	public Result<BaseArchivesCollectionLine> queryById(@RequestParam(name="id",required=true) String id) {
		Result<BaseArchivesCollectionLine> result = new Result<BaseArchivesCollectionLine>();
		BaseArchivesCollectionLine baseArchivesCollectionLine = baseArchivesCollectionLineService.getById(id);
		if(baseArchivesCollectionLine==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(baseArchivesCollectionLine);
			result.setSuccess(true);
		}
		return result;
	}
}
