package org.jeecg.modules.invoice.entity;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;
import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import com.fasterxml.jackson.annotation.JsonFormat;
import org.jeecg.common.dto.invoice.PurchaseAddReqDTO;
import org.springframework.format.annotation.DateTimeFormat;
import javax.validation.constraints.NotEmpty;

/**
 * @Description: 采购,销售发票
 * @author： jeecg-boot
 * @date：   2021-02-26
 * @version： V1.0
 */
@ApiModel("采购,销售发票")
@Data
@TableName("invoice_manage_purchase")
public class InvoiceManagePurchase implements Serializable {
	private static final long serialVersionUID = 4097348073460739726L;
	@ApiModelProperty("id")
	@TableId(type = IdType.UUID)
	private String id;
	@ApiModelProperty("单据编号")
	private String billcode;
	@ApiModelProperty("类型（1.采购发票 2.销售发票）")
	@NotEmpty(message = "参数type不能为空")
	private String type;
	@ApiModelProperty("项目档案id")
	private String proArchivesId;
	@NotEmpty(message = "参数proCode不为空")
	@ApiModelProperty("项目编码")
	private String proCode;
	@ApiModelProperty("项目名称")
	@NotEmpty(message = "参数proName不为空")
	private String proName;
	@ApiModelProperty("填写人(当前用户)")
	@TableField(value = "`current_user`")
	private String currentUser;
	@ApiModelProperty("创建时间")
	@JsonFormat(timezone = "GMT+8",pattern = "yyyy-MM-dd HH:mm:ss")
    @DateTimeFormat(pattern="yyyy-MM-dd HH:mm:ss")
	private Date createTime;
	@ApiModelProperty("创建人")
	private String createBy;
	@ApiModelProperty("修改时间")
	@JsonFormat(timezone = "GMT+8",pattern = "yyyy-MM-dd HH:mm:ss")
    @DateTimeFormat(pattern="yyyy-MM-dd HH:mm:ss")
	private Date updateTime;
	@ApiModelProperty("修改人")
	private String updateBy;
	@ApiModelProperty("删除状态（0，正常，1已删除）")
	private String delFlag;
	@ApiModelProperty("组织")
	private String pkOrg;

	public InvoiceManagePurchase() {
	}

	public InvoiceManagePurchase(PurchaseAddReqDTO reqDTO) {
		this.id = reqDTO.getId();
		this.billcode = reqDTO.getBillcode();
		this.type = reqDTO.getType();
		this.proArchivesId = reqDTO.getProArchivesId();
		this.proCode = reqDTO.getProCode();
		this.proName = reqDTO.getProName();
		this.currentUser = reqDTO.getCurrentUser();
		this.delFlag = "0";
		this.pkOrg = reqDTO.getPkOrg();
	}
}
