package org.jeecg.modules.invoice.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.google.common.collect.Lists;
import org.jeecg.common.dto.invoice.PurchaseAddReqDTO;
import org.jeecg.common.dto.invoice.PurchaseDetailAddReqDTO;
import org.jeecg.common.dto.invoice.PurchaseDetailListRespDTO;
import org.jeecg.common.dto.invoice.PurchaseRespDTO;
import org.jeecg.modules.invoice.entity.InvoiceManagePurchase;
import org.jeecg.modules.invoice.entity.InvoiceManagePurchaseDetail;
import org.jeecg.modules.invoice.mapper.InvoiceManagePurchaseMapper;
import org.jeecg.modules.invoice.service.InvoiceManagePurchaseDetailService;
import org.jeecg.modules.invoice.service.InvoiceManagePurchaseService;
import org.jeecg.modules.system.service.ISysSerialPatternService;
import org.jeecg.modules.system.vo.CallResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.transaction.annotation.Transactional;
import java.util.List;

/**
 * @Description: 采购,销售发票
 * @author： jeecg-boot
 * @date：   2021-02-26
 * @version： V1.0
 */
@Service
@Transactional
public class InvoiceManagePurchaseServiceImpl extends ServiceImpl<InvoiceManagePurchaseMapper, InvoiceManagePurchase> implements InvoiceManagePurchaseService {
    @Autowired
    private ISysSerialPatternService sysSerialPatternService;
    @Autowired
    private InvoiceManagePurchaseDetailService invoiceManagePurchaseDetailService;

     /**
       * @Author jihaosen
       * @date 2021/3/2
       * @desc 新增采购,销售发票
       */

    @Override
    public void add(PurchaseAddReqDTO reqDTO) {

        InvoiceManagePurchase invoiceManagePurchase = new InvoiceManagePurchase(reqDTO);
        // 设置编码
        CallResult<String> nextSerial = sysSerialPatternService.getNextSerial(invoiceManagePurchase, "invoice_manage_purchase", "billcode", true);
        if(!nextSerial.isSucceed()){
            throw new RuntimeException("获取编号失败");
        }
        invoiceManagePurchase.setBillcode(nextSerial.getContent());
        int sort = 1;
        save(invoiceManagePurchase);
        for(PurchaseDetailAddReqDTO detail : reqDTO.getDetailList()){
            detail.setPurchaseId(invoiceManagePurchase.getId());
            InvoiceManagePurchaseDetail invoiceManagePurchaseDetail = new InvoiceManagePurchaseDetail(detail);
            invoiceManagePurchaseDetail.setSort(sort++);
            invoiceManagePurchaseDetailService.save(invoiceManagePurchaseDetail);
        }

    }

    /**
     * @Author jihaosen
     * @date 2021/3/2
     * @desc 编辑采购,销售发票
     */
    @Override
    public boolean edit(PurchaseAddReqDTO reqDTO) {
        InvoiceManagePurchase invoiceManagePurchase = new InvoiceManagePurchase(reqDTO);
        updateById(invoiceManagePurchase);
        //删除子表
        deleteChildrenTable(invoiceManagePurchase.getId());
         //新增子表数据
        for(PurchaseDetailAddReqDTO detail : reqDTO.getDetailList()){
            detail.setPurchaseId(invoiceManagePurchase.getId());
            InvoiceManagePurchaseDetail invoiceManagePurchaseDetail = new InvoiceManagePurchaseDetail(detail);
            invoiceManagePurchaseDetailService.save(invoiceManagePurchaseDetail);
        }

        return true;
    }

    /**
     * @Author jihaosen
     * @date 2021/3/2
     * @desc 删除采购,销售发票
     */
    @Override
    public boolean deleteById(InvoiceManagePurchase invoiceManagePurchase) {
        invoiceManagePurchase.setDelFlag("1");
        updateById(invoiceManagePurchase);
        //删除子表
        deleteChildrenTable(invoiceManagePurchase.getId());
        return true;
    }

    /**
     * @desc 根据id查询
     */
    @Override
    public PurchaseRespDTO queryById(String id) {
        InvoiceManagePurchase invoiceManagePurchase = getById(id);
        PurchaseRespDTO respDTO = new PurchaseRespDTO(invoiceManagePurchase);

        //获取子数据
        QueryWrapper<InvoiceManagePurchaseDetail> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("del_flag", "0");
        queryWrapper.eq("purchase_id", id);
        queryWrapper.orderByAsc("sort");
        List<InvoiceManagePurchaseDetail> details  = invoiceManagePurchaseDetailService.list(queryWrapper);
        List<PurchaseDetailListRespDTO> list = Lists.newArrayList();
        for(InvoiceManagePurchaseDetail detail : details){
            PurchaseDetailListRespDTO detailResp = new PurchaseDetailListRespDTO(detail);
            list.add(detailResp);
        }
        respDTO.setDetailList(list);
        return respDTO;
    }



    /**
     * @desc 删除子表数据
     */
    private void deleteChildrenTable(String id) {
        QueryWrapper<InvoiceManagePurchaseDetail> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("purchase_id", id).eq("del_flag", "0");
        invoiceManagePurchaseDetailService.update(new InvoiceManagePurchaseDetail("1"), queryWrapper);
    }
}
