package org.jeecg.modules.payment.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.google.common.collect.Lists;
import org.jeecg.common.dto.archives.ExpensePriceReqDTO;
import org.jeecg.common.dto.archives.ExpensePriceRespDTO;
import org.jeecg.common.dto.payment.ProjectArchiveReqDTO;
import org.jeecg.common.dto.payment.ProjectArchiveRespDTO;
import org.jeecg.common.dto.payment.SlipAddReqDTO;
import org.jeecg.common.dto.payment.SlipContentAddReqDTO;
import org.jeecg.common.dto.payment.SlipContentRespDTO;
import org.jeecg.common.dto.payment.SlipRespDTO;
import org.jeecg.modules.archives.entity.ProjectManageArchives;
import org.jeecg.modules.archives.service.ProjectManageArchivesService;
import org.jeecg.modules.payment.entity.ManagePaymentAndReceiptContent;
import org.jeecg.modules.payment.entity.ManagerPaymentAndReceiptSlip;
import org.jeecg.modules.payment.mapper.ManagerPaymentAndReceiptSlipMapper;
import org.jeecg.modules.payment.service.ManagePaymentAndReceiptContentService;
import org.jeecg.modules.payment.service.ManagerPaymentAndReceiptSlipService;
import org.jeecg.modules.system.service.ISysSerialPatternService;
import org.jeecg.modules.system.vo.CallResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.transaction.annotation.Transactional;
import java.math.BigDecimal;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @Description: 收付款管理--收款单
 * @author： jeecg-boot
 * @date：   2021-03-03
 * @version： V1.0
 */
@Service
@Transactional
public class ManagerPaymentAndReceiptSlipServiceImpl extends ServiceImpl<ManagerPaymentAndReceiptSlipMapper, ManagerPaymentAndReceiptSlip> implements ManagerPaymentAndReceiptSlipService {

    @Autowired
    private ISysSerialPatternService sysSerialPatternService;
    @Autowired
    private ManagePaymentAndReceiptContentService managePaymentAndReceiptContentService;
    @Autowired
    private ManagerPaymentAndReceiptSlipMapper managerPaymentAndReceiptSlipMapper;
    @Autowired
    private ProjectManageArchivesService projectManageArchivesService;

    /**
     * @desc 新增
     */
    @Override
    public void add(SlipAddReqDTO reqDTO) {
        ManagerPaymentAndReceiptSlip slip = new ManagerPaymentAndReceiptSlip(reqDTO);
        //设置编码
        CallResult<String> nextSerial = sysSerialPatternService.getNextSerial(slip, "manager_payment_and_receipt_slip", "billcode", true);
        if(!nextSerial.isSucceed()){
            throw new RuntimeException("获取编号失败");
        }
        slip.setBillcode(nextSerial.getContent());
        save(slip);
        //添加子表
        saveChildren(reqDTO, slip.getId());
    }

    /**
     * @desc 编辑
     */
    @Override
    public boolean edit(SlipAddReqDTO reqDTO) {
        ManagerPaymentAndReceiptSlip slip = new ManagerPaymentAndReceiptSlip(reqDTO);
            updateById(slip);
       //删除子表
        deleteChildren(slip.getId());
        saveChildren(reqDTO, slip.getId());
        return true;
    }

    /**
     * @desc 通过id查询
     */
    @Override
    public SlipRespDTO queryById(String id) {
        ManagerPaymentAndReceiptSlip  slip = getById(id);
        SlipRespDTO respDTO = new SlipRespDTO(slip);
        List<SlipContentRespDTO> respDTOS = getChildren(id);
        respDTO.setDetailList(respDTOS);
        return respDTO;
    }

    /**
     * @desc 通过id删除
     */
    @Override
    public Boolean dropById(ManagerPaymentAndReceiptSlip managerPaymentAndReceiptSlip) {
        managerPaymentAndReceiptSlip.setDelFlag("1");
        updateById(managerPaymentAndReceiptSlip);
        //删除子表
        deleteChildren(managerPaymentAndReceiptSlip.getId());
        return true;
    }

    /**
     * @desc 统计汇款单金额
     */
    @Override
    public ExpensePriceRespDTO getExpensePrice(ExpensePriceReqDTO reqDTO) {
       List<ManagePaymentAndReceiptContent> contents = managerPaymentAndReceiptSlipMapper.getExpensePrice(reqDTO);
        BigDecimal price = BigDecimal.ZERO;
        for(ManagePaymentAndReceiptContent content : contents){
            price = price.add(content.getCoPrvice());
        }
        return new ExpensePriceRespDTO(price);
    }

    /**
     * @desc 获取项目档案的名称与编码
     */
    @Override
    public List<ProjectArchiveRespDTO> getProNameAndCode(ProjectArchiveReqDTO reqDTO) {
        //获取收付款单列表数据
        QueryWrapper< ManagerPaymentAndReceiptSlip> slipQueryWrapper = new QueryWrapper<>();
        slipQueryWrapper.eq("del_flag", "0");
        slipQueryWrapper.eq("pk_org", reqDTO.getPkOrg());
        List<ManagerPaymentAndReceiptSlip> slips = list(slipQueryWrapper);
        List<String> proIdList = slips.stream().map(ManagerPaymentAndReceiptSlip :: getProId).collect(Collectors.toList());
        //获取项目档案列表数据
        QueryWrapper<ProjectManageArchives> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("del_flag", "0");
        queryWrapper.eq("pk_Org", reqDTO.getPkOrg());
        List<ProjectManageArchives> projectManageArchives = projectManageArchivesService.list(queryWrapper);
        //过滤已使用的项目档案
        List<ProjectArchiveRespDTO> respDTOS = Lists.newArrayList();
        for(ProjectManageArchives archives : projectManageArchives){
            if(!proIdList.contains(archives.getId())){
                respDTOS.add(new ProjectArchiveRespDTO(archives.getId(), archives.getProCode(), archives.getProName()));
            }
        }
        return respDTOS;
    }



    /**
     * @desc 获取子表数据
     */
    private List<SlipContentRespDTO> getChildren(String id) {
        QueryWrapper<ManagePaymentAndReceiptContent> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("del_flag", "0");
        queryWrapper.eq("slip_id", id);
        queryWrapper.orderByAsc("sort");
        List<ManagePaymentAndReceiptContent> list = managePaymentAndReceiptContentService.list(queryWrapper);
        List<SlipContentRespDTO> respDTOS = Lists.newArrayList();
        for(ManagePaymentAndReceiptContent content : list){
            respDTOS.add(new SlipContentRespDTO(content));
        }
        return  respDTOS;
    }

    //删除子表
    private void deleteChildren(String id) {
        QueryWrapper<ManagePaymentAndReceiptContent> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("del_flag", "0");
        queryWrapper.eq("slip_id", id);
        managePaymentAndReceiptContentService.update(new ManagePaymentAndReceiptContent("1"), queryWrapper);

    }

    /**
     * @desc 添加子表
     */
    private void saveChildren(SlipAddReqDTO slipAddReqDTO, String id) {
        List<ManagePaymentAndReceiptContent> list = Lists.newArrayList();
        int sort = 1;
        for(SlipContentAddReqDTO reqDTO : slipAddReqDTO.getDetailList()){
            ManagePaymentAndReceiptContent content = new ManagePaymentAndReceiptContent(reqDTO);
            content.setSlipId(id);
            content.setSort(sort);
            list.add(content);
        }
        managePaymentAndReceiptContentService.saveBatch(list);
    }
}
