package org.jeecg.modules.prowork.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Lists;
import io.swagger.models.auth.In;
import org.apache.commons.lang3.StringUtils;
import org.jeecg.common.dto.prowork.ProWorkMilestoneAddReqDTO;
import org.jeecg.common.dto.prowork.ProWorkMilestoneRespDTO;
import org.jeecg.common.dto.prowork.ProWorkUpdateFileAddReqDTO;
import org.jeecg.common.dto.prowork.ProWorkUpdateFileRespDTO;
import org.jeecg.common.exception.JeecgBootException;
import org.jeecg.modules.prowork.entity.ProWorkMilestone;
import org.jeecg.modules.prowork.entity.ProWorkUpdateFile;
import org.jeecg.modules.prowork.mapper.ProWorkMilestoneMapper;
import org.jeecg.modules.prowork.service.ProWorkMilestoneService;
import org.jeecg.modules.prowork.service.ProWorkUpdateFileService;
import org.jeecg.modules.system.service.ISysSerialPatternService;
import org.jeecg.modules.system.vo.CallResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import java.util.List;

/**
 * @Description: 项目工作--里程碑确认单
 * @author： jeecg-boot
 * @date：   2021-03-03
 * @version： V1.0
 */
@Service
@Transactional
public class ProWorkMilestoneServiceImpl extends ServiceImpl<ProWorkMilestoneMapper, ProWorkMilestone> implements ProWorkMilestoneService {
    @Autowired
    private ISysSerialPatternService sysSerialPatternService;
    @Autowired
    private ProWorkUpdateFileService proWorkUpdateFileService;

    /**
     * @desc 新增
     */
    @Override
    public void add(ProWorkMilestoneAddReqDTO reqDTO) {
        ProWorkMilestone proWorkMilestone = new ProWorkMilestone(reqDTO);
        CallResult<String> nextSerial = sysSerialPatternService.getNextSerial(proWorkMilestone, "pro_work_milestone", "billcode", true);
        if(!nextSerial.isSucceed()){
            throw new RuntimeException("获取编号失败");
        }
        proWorkMilestone.setBillcode(nextSerial.getContent());
        save(proWorkMilestone);
        //保存文件
        saveChildren(reqDTO, proWorkMilestone.getId());
    }

    /**
     * @desc 编辑
     */
    @Override
    public boolean edit(ProWorkMilestoneAddReqDTO reqDTO) {
        ProWorkMilestone proWorkMilestone= new ProWorkMilestone(reqDTO);
        updateById(proWorkMilestone);
        //删除子表
        deleteChildren(reqDTO.getId());
        //添加子表
        saveChildren(reqDTO, proWorkMilestone.getId());
        return true;
    }

    /**
     * @desc 通过id删除
     */
    @Override
    public boolean dropById(ProWorkMilestone proWorkMilestone) {
        proWorkMilestone.setDelFlag("1");
        updateById(proWorkMilestone);

        //删除子表
        deleteChildren(proWorkMilestone.getId());
        return true;
    }

    /**
     * @desc 通过id查询
     */
    @Override
    public ProWorkMilestoneRespDTO queryById(String id) {
        ProWorkMilestone proWorkMilestone = getById(id);
        ProWorkMilestoneRespDTO proWorkMilestoneRespDTO = new ProWorkMilestoneRespDTO(proWorkMilestone);
        List<ProWorkUpdateFileRespDTO> fileList = getChildern(id);
        proWorkMilestoneRespDTO.setFileList(fileList);
        return proWorkMilestoneRespDTO;
    }



    /**
     * @desc 获取子表数据（文件信息）
     */
    private List<ProWorkUpdateFileRespDTO> getChildern(String id) {
        QueryWrapper<ProWorkUpdateFile> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("work_mile_id", id);
        queryWrapper.lambda().eq(ProWorkUpdateFile::getDelFlag,"0");
        queryWrapper.orderByAsc("sort");
        List<ProWorkUpdateFile> list = proWorkUpdateFileService.list(queryWrapper);
        List<ProWorkUpdateFileRespDTO> fileList = Lists.newArrayList();
        for(ProWorkUpdateFile updateFile : list){
            ProWorkUpdateFileRespDTO respDTO = new ProWorkUpdateFileRespDTO(updateFile);
            fileList.add(respDTO);
        }
        return fileList;
    }

    /**
     * @desc 删除子表
     */
    private void deleteChildren(String id) {
        QueryWrapper<ProWorkUpdateFile> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("del_flag", "0").eq("work_mile_id",id);
        proWorkUpdateFileService.update(new ProWorkUpdateFile("1"), queryWrapper);

    }

    //保存文件
    private void saveChildren(ProWorkMilestoneAddReqDTO reqDTO, String id) {
        if(CollectionUtils.isEmpty(reqDTO.getFileList())){
            throw new JeecgBootException("文件信息不能为空");
        }
        List<ProWorkUpdateFile> list = Lists.newArrayList();
        int sort = 1;
        for(ProWorkUpdateFileAddReqDTO fileAddReqDTO : reqDTO.getFileList()){
            ProWorkUpdateFile proWorkUpdateFile = new ProWorkUpdateFile(fileAddReqDTO);
            //截取文件名称
            String[] filePath = fileAddReqDTO.getFileUrl().split("/");
            int index = filePath.length;
            String fileNameAfter = StringUtils.substringAfter(filePath[index - 1], ".");
            String fileNameBefore = StringUtils.substringBefore(filePath[index - 1], "_");
            String fileName = fileNameBefore + "." + fileNameAfter;
            proWorkUpdateFile.setFileName(fileName);
            proWorkUpdateFile.setWorkMileId(id);
            proWorkUpdateFile.setSort(sort++);
            list.add(proWorkUpdateFile);
        }
        proWorkUpdateFileService.saveBatch(list);
    }
}
